/*
 * This library provides the functionnality of reading NFC tags 
 * using J2ME.
 *
 * Copyright (C) 2009-2011, DECARTE (INRIA)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library in the file LICENSE.TXT; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */


import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import javax.microedition.contactless.ContactlessException;
import javax.microedition.contactless.DiscoveryManager;
import javax.microedition.contactless.TargetProperties;
import javax.microedition.contactless.TargetListener;
import javax.microedition.contactless.TargetType;

// Packages for NDEF
import javax.microedition.contactless.ndef.NDEFRecord;
import javax.microedition.contactless.ndef.NDEFMessage;
import javax.microedition.contactless.ndef.NDEFRecordType;
import javax.microedition.contactless.ndef.NDEFTagConnection;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;


// Packages for GUI Stuff
import javax.microedition.lcdui.Alert;
import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Choice;
import javax.microedition.lcdui.ChoiceGroup;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Form;
import javax.microedition.lcdui.StringItem;
import javax.microedition.lcdui.TextBox;

import javax.microedition.midlet.*;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

public class ALEReader extends MIDlet implements TargetListener, CommandListener,Runnable {

    // Display Stuff
    private Command exitCommand;
    private Form form;
    private ChoiceGroup choiseGroup;
    private int LastString;
    
    private String Received = "fileconn.dir.received";
    private String path;
    boolean ALEisDefined = true;
    boolean ALEWorks = false;
    
    Vector includeKeys = new Vector();
    Vector excludeKeys = new Vector();
    
    int FilterMode = 0; // 0:Include     1: Exclude
    
    public ALEReader() {
        exitCommand = new Command("Exit", Command.EXIT, 1);
        form = new Form("NFC Light ALE");

        form.addCommand(exitCommand);
        form.setCommandListener(this);
        form.append("Touch Tag to Read infos");

         // Registration of the TargetListener for external contactless
        // Targets (in this RFID_TAG).
        try {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.addTargetListener(this, TargetType.NDEF_TAG);
            
        } catch (ContactlessException ce) {
            displayAlert("Unable to register TargetListener: " + ce.toString(), AlertType.ERROR);
        }

    }

    public void startApp() {
    	 Display.getDisplay(this).setCurrent(form);
         (new Thread(this)).start();
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {

    }

    
 // read XML and parse
    public void run() {
        try {
        	InputStream in = null;
            // this will handle our XML
        	XmlHandler Xmlhandler = new XmlHandler();
 
            // get a parser object
            SAXParser parser = SAXParserFactory.newInstance().newSAXParser();
            // InputStream in = getClass().getResourceAsStream("/EcSpec.xml");// Used in the case when EcSpec.xml is in the jar
            
            path = System.getProperty(Received)+"ECSpec.xml";
     	   try {
     	      FileConnection filecon = (FileConnection)
     	        Connector.open(path, Connector.READ);
     	       
     	      if(!filecon.exists()) {
     	    	  form.append("\n No ECSpec file found, No ALE applied!");
     	    	 ALEisDefined = false;
     	      }
     	      else {  in = filecon.openInputStream();
     	      		filecon.close();}
     	   } catch(IOException ioe) {
     	   }
     	   
     	   if (ALEisDefined){
            try {
            	
                // parse the XML data stream
                parser.parse(in, Xmlhandler);
            }  catch (Exception e) {
            	 form.append("\n  ECSpec file missing or errors.");
            	 return;
            }
            try {
                in.close();
            }  catch (Exception e) {
            }
            
            
            Vector includes, excludes;
            includes = Xmlhandler.getInclude();
            excludes = Xmlhandler.getExclude();
            
            
            int result = getKeys(includes,excludes,includeKeys,excludeKeys );
            if (result==0)
            {ALEWorks =true;
            
            form.append("\n      -- ALE filering --");
            
            if (includeKeys.size()==0)
            	FilterMode =1;
            if (excludeKeys.size()==0)
            	FilterMode =0;
            	
            int length;
            length = includes.size();
            form.append("\n   -- IncludePattern: --\n");
            for (int i = 0; i<length; i++)
            	{
            		String tempo = (String) includes.elementAt(i);
            		form.append(tempo+"\n");
            	}
            length = excludes.size();
            form.append("\n  -- ExcludePattern: --\n");
            for (int j = 0; j<length; j++)
            	{
            		String tempo2 = (String) excludes.elementAt(j);
            		form.append(tempo2+"\n");
            	}
          
            
            }  
            }
     	  form.append("    \n");
     	  form.append("  -- The current tag -- \n");
     	  LastString = form.append("");
     	}catch (Exception e) {
            form.append(e.toString());
        }
    
    }
    
    
    public int getKeys(Vector includes, Vector excludes, Vector includeKeys, Vector excludeKeys ){
    	int result = 0;
    	int i,j;
    	
    	int length;
    	String[] elements= {"","","","","",""};
        length = includes.size();
        for ( i = 0; i<length; i++)
        	{
        		String tempo = (String) includes.elementAt(i);
        		try{
                    elements = split(tempo);
                    includeKeys.addElement(elements[4]);
                    }
                   catch (Exception e) {
                	   result = -1;
                	   form.append("\n  ECSpec file format errors.");
                   }
        	}
        
        String[] elements2= {"","","","","",""};
        length = excludes.size();
        for ( j = 0; j<length; j++)
        	{
        		String tempo2 = (String) excludes.elementAt(j);
        		try{
                    elements2 = split(tempo2);
                    excludeKeys.addElement(elements2[4]);
                    }
                   catch (Exception e) {
                	   result = -1;
                	   form.append("\n  ECSpec file format errors.");
                   }
        	}
        
        if ((i==0)&&(j==0))
        	result = -1;
        		
		return result;
    	 }
    /**
     * Implementation of the Call-Back Function of the TargetListener
     * @param targetProperties: Array of Targets found by the Phone
     *
     */
    public void targetDetected(TargetProperties[] targetProperties) {

    	boolean TagPassFiltering;
        // in case no targets found, exit the method
        if (targetProperties.length == 0) {
            return;
        }

       
        NDEFTagConnection ndconn = getNDEFTAGConnection(targetProperties);
        
        try {
        	byte[] tempo = ndconn.readNDEF().getRecord(0).getPayload();
            String value = new String(tempo);
            
            if (ALEWorks == false)
            {
            	 form.delete(LastString);
                 LastString = form.append(value);
            }
            else{
            String[] elements= {"","","","","",""};
            try{
             elements = split(value);
            String Key = elements[4];
            
            if (FilterMode==0){ // Include mode
            	  TagPassFiltering = false;
            	 int length = includeKeys.size();
                 for (int i = 0; i<length; i++)
                 	{
                 	  if (Key.equalsIgnoreCase((String)includeKeys.elementAt(i)))
                 	  {
                 		  TagPassFiltering = true;
                 		  break;
                 	  }
                 	}
            	}
            else { // Exclude mode
            	 TagPassFiltering = true;
            	int length2 = excludeKeys.size();
                for (int j = 0; j<length2; j++)
                	{
                	  if (Key.equalsIgnoreCase((String)excludeKeys.elementAt(j)))
                	  {
                		  TagPassFiltering = false;
                		  break;
                	  }
                	}
            }
            
            if (TagPassFiltering){
            	 form.delete(LastString);
            	 LastString = form.append(value);
             } else{
            	 form.delete(LastString);
            	 LastString = form.append("");
            	 displayAlert("The current tag: "+value+ " is filtered out by the ALE! ", AlertType.INFO);
             }
            	 
            
            	
           
             }
            catch (Exception e) {}
            
          
            }

        } catch (IOException io) {
           // displayAlert("IOException during open() or Write(): " + io.toString(), AlertType.ERROR);
        } catch (ContactlessException io) {
            //displayAlert("ContactlessException during writeNDEF(): " + io.toString(), AlertType.ERROR);
        } catch (Exception e) {
            //displayAlert("Exception: " + e.toString(), AlertType.ERROR);
        } // in case of an exception, close connection properly
        finally {

            // close Connection again
            if (ndconn != null) {
                try {
                    ndconn.close();
                } catch (IOException ex) {
                    displayAlert("IOException during close(): " + ex.toString(), AlertType.ERROR);
                }
            }
        }

    }
    
    private String[] split(String original) {
    	Vector nodes = new Vector();
    	String separator = ":";
    	System.out.println("split start...................");
    	// Parse nodes into vector
    	int index = original.indexOf(separator);
    	while(index>=0) {
    	nodes.addElement( original.substring(0, index) );
    	original = original.substring(index+separator.length());
    	index = original.indexOf(separator);
    	}
    	// Get the last node
    	nodes.addElement( original );

    	// Create splitted string array
    	String[] result = new String[ nodes.size() ];
    	if( nodes.size()>0 ) {
    	for(int loop=0; loop<nodes.size(); loop++)
    	{
    	result[loop] = (String)nodes.elementAt(loop);
    	System.out.println(result[loop]);
    	}

    	}

    	return result;
    	}
    

    private NDEFTagConnection getNDEFTAGConnection(TargetProperties[] tProp) {
        for (int j = 0; j < tProp.length; j++) {
            Class[] connections = tProp[j].getConnectionNames();
            if (connections != null) {
                for (int i = 0; i < connections.length; i++) {
                    if (connections[i].getName().equals(
                            "javax.microedition.contactless.ndef.NDEFTagConnection")) {
                        try {
                            return (NDEFTagConnection) Connector.open(tProp[0].getUrl(connections[i]));
                        } catch (Exception e) {
                            displayAlert("Error writting NDEF: " + e.toString(), AlertType.ERROR);
                        }
                    }
                }
            }
        }
        return null;
    }

    /**
     * Implementation of the Call-Back function of the CommandListener
     * @param command: command key pressed
     * @param displayable: associated displayable Object
     */
    public void commandAction(Command command, Displayable displayable) {
        if (command == exitCommand) {
            DiscoveryManager dm = DiscoveryManager.getInstance();
            dm.removeTargetListener(this, TargetType.NDEF_TAG);
            destroyApp(false);
            notifyDestroyed();
        }
    }

    private void displayAlert(String error, AlertType type) {
        Alert err = new Alert(form.getTitle(), error, null, type);
        Display.getDisplay(this).setCurrent(err, form);
    }
}
